<?php
/**
 * ImageManager Provider - Full Database
 *
 * @author 		R2H - Michael Snoeren
 * @link 		https://r2h.nl/
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// Deny direct access
defined('_JEXEC') or die;

class PlgImproviderFullDb extends JPlugin
{
	/**
	 * Autoload the languages
	 * @var 	Boolean
	 * @access 	Protected
	 */
	protected $autoloadLanguage = true;
	protected $images 			= [];

	protected $modelDb 		= null;

	/**
	 * Provide basic data about your plugin
	 * @access 	Public
	 * @return 	Array
	 */
	public function onProviderIdentify()
	{
		return [
			'title' 			=> JText::_('PLG_IMPROVIDER_FULLDB_TITLE'),
			'author' 			=> JText::_('PLG_IMPROVIDER_FULLDB_AUTHOR'),
			'description' 		=> JText::_('PLG_IMPROVIDER_FULLDB_DESCRIPTION'),
			'version' 			=> '',
			'createdOn' 		=> '17-10-2017',
			'license' 			=> 'GNU/GPLv3'
		];
	}

	/**
	 * Prepare the list of images
	 * @param 	Array 	$images 	The image array with relative paths from the server root
	 * @access 	Public
	 * @return 	Void
	 */
	public function onProviderPrepare($images)
	{
		JLoader::register('PlgIMProviderFullDbModelDb', __DIR__ . '/models/db.php');

		if (is_null($this->modelDb)) {
			$this->modelDb 		= JModelLegacy::getInstance('Db', 'PlgImproviderFullDbModel');
		}

		$this->images 			= $this->modelDb->findUsage($images);
	}

	/**
	 * Get all used images
	 * @access 	Public
	 * @return 	Array
	 */
	public function onImagesGetUsed()
	{
		return array_filter($this->images);
	}
	/**
	 * Get all unused images
	 * @access 	Public
	 * @return 	Array
	 */
	public function onImagesGetUnused()
	{
		return array_filter($this->images, function ($var) {
			return $var === false;
		});
	}

	/**
	 * Get the usage location of a given path
	 * @param 	String 	$path 	Relative image path
	 * @access 	Public
	 * @return 	Array|Null
	 */
	public function onImageGetLocations($path)
	{
		JLoader::register('PlgIMProviderFullDbModelDb', __DIR__ . '/models/db.php');
		$model 					= JModelLegacy::getInstance('Db', 'PlgImproviderFullDbModel');

		// Find the location of this image
		$locations 				= $model->findLocation($path);
		$locationArray 			= [];

		foreach ($locations as $loc) {
			$locationArray[] 	= [
				'type' 			=> JText::_('PLG_IMPROVIDER_FULLDB_TYPE'),
				'title' 		=> $loc,
				'status' 		=> ImageManagerHelperStatus::PLUGIN_CANNOTPROCESS,
				'url' 			=> ''
			];
		}

		return $locationArray;
	}

	/**
	 * Replace the path of an image with a new one
	 * @param 	Array 	$images Key is the old path, value is the new path
	 * @access 	Public
	 * @return 	Boolean
	 */
	public function onImagesChangeLocation(array $images)
	{
		return true;
	}
}
