<?php
/**
 * ImageManager Provider - Full Database
 *
 * @author 		R2H - Michael Snoeren
 * @link 		https://r2h.nl/
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// Deny direct access
defined('_JEXEC') or die;

class PlgIMProviderFullDbModelDb extends JModelLegacy
{
	/**
	 * Model Data
	 * @var 	Mixed
	 * @access 	Protected
	 */
	protected $tables 			= [];

	/**
	 * Constructor
	 * @param 	Array 	$config
	 * @access 	Public
	 * @return 	Void
	 */
	public function __construct($config = [])
	{
		parent::__construct($config);

		// Setup the table/columns index
		$db 					= $this->getDbo();
		$tables 				= $db->getTableList();

		foreach ($tables as $table) {
			$this->tables[$table] = array_keys($db->getTableColumns($table));
		}

		// Remove the last location table
		foreach ($this->tables as $table => $cols) {
			if (strpos($table, '_im_location')) {
				unset($this->tables[$table]);
			}
		}
	}

	/**
	 * Check where images are being used
	 * @param 	Array 	$images
	 * @access 	Public
	 * @return 	Array
	 */
	public function findUsage(array $images)
	{
		$db 					= $this->getDbo();
		$return 				= [];
		$imageChunks 			= array_chunk($images, 25);

		foreach ($imageChunks as $imageList) {
			foreach ($this->tables as $table => $columns) {
				$query 			= $this->getQuery($table, $imageList);
				$db->setQuery($query);

				$result 		= $db->loadObjectList();

				// No results found, populate return array
				if (!count($result)) {
					foreach ($imageList as $img) {
						if (!array_key_exists($img, $return)) {
							$return[$img] = ImageManagerHelperStatus::STATUS_UNUSED;
						}
					}

					continue 1;
				}

				foreach ($columns as $col) {
					$resultList = [];

					foreach ($result as $r) {
						$resultList[] = $r->{$col};
					}

					foreach ($imageList as $img) {
						if (!array_key_exists($img, $return)) {
							$return[$img] = ImageManagerHelperStatus::STATUS_UNUSED;
						}

						foreach ($resultList as $r) {
							if (strpos($r, $img) !== false ||
								strpos($r, str_replace('/', '\\/', $img)) !== false) {
								$return[$img] = ImageManagerHelperStatus::STATUS_USEDNOPROCESSING;
							}
						}
					}
				}
			}
		}

		return $return;
	}

	/**
	 * Find the location of the image in the database
	 * @param 	String 	$image
	 * @access 	Public
	 * @return 	Array
	 */
	public function findLocation($image)
	{
		if (empty($image)) {
			return [];
		}

		$db 					= $this->getDbo();
		$return 				= [];

		foreach ($this->tables as $table => $cols) {
			$query 				= $this->getQuery($table, [$image]);
			$db->setQuery($query);

			$result 			= $db->loadObjectList();

			if (!count($result)) {
				continue;
			}

			$return[] 			= $table;
		}

		return $return;
	}

	/**
	 * Get a query object that searches in a given table
	 * @param 	String 	$table
	 * @param 	Array 	$images
	 * @access 	Protected
	 * @return 	Object
	 */
	protected function getQuery($table, array $images)
	{
		if (!is_string($table) || !array_key_exists($table, $this->tables)) {
			return null;
		}

		$db 					= $this->getDbo();
		$query 					= $db->getQuery(true);

		$query
		->select('*')
		->from($db->qn($table));

		foreach ($images as $img) {
			foreach ($this->tables[$table] as $col) {
				// Search for image paths and escaped variants
				$query->where($db->qn($col) . ' LIKE ' . $db->q('%' . $img . '%'), 'OR');
				$query->where(
					'REPLACE(' . $db->quoteName($col) . ', "\\\\/", "/") LIKE ' . $db->q('%' . $img . '%'),
					'OR'
				);
			}
		}

		return $query;
	}
}
